* Output options - no page number, date, centering, or page breaks, 
  page length=Maximum and width=120,  eliminate default titles;
OPTIONS nonumber nodate nocenter formdlim=" " pagesize=MAX linesize=120; TITLE; 

* Library for data files;
* Replace path with location of .sasb7sat file;
LIBNAME folder "F:\Example Data\Locker_Hoffman_Bovaird_Electronic_Appendix";

*******************************************************************************************;
* To use TotalR2 macro:
PredFewer = name of dataset of predicted outcomes for nested model
NameFewer = label to use for nested model 
PredMore = 	name of dataset of predicted outcomes for comparison model
NameMore = 	label to use for comparison model;

%MACRO TotalR2(DV, PredFewer, NameFewer, PredMore, NameMore);
PROC CORR NOPRINT NOSIMPLE DATA=&PredFewer. OUTP=CorrFewer; VAR pred &DV.; RUN;
PROC CORR NOPRINT NOSIMPLE DATA=&PredMore.  OUTP=CorrMore;  VAR pred &DV.; RUN;
DATA CorrFewer; LENGTH Name $30.; SET CorrFewer; Name="&NameFewer."; RUN;
DATA CorrMore;  LENGTH Name $30.; SET CorrMore;  Name="&NameMore.";  RUN;
DATA CorrCompare; LENGTH Name $30.; SET CorrFewer CorrMore; 
	PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
	IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; DROP Pred; RUN;
DATA CorrCompare; SET CorrCompare; TotalR2Diff=TotalR2-LAG1(TotalR2);
	KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
TITLE2 "Total R2 (% Reduction) for &NameFewer. vs. &NameMore.";
PROC PRINT NOOBS DATA=CorrCompare; RUN; TITLE2;	
%MEND TotalR2;

* To use PseudoR2 macro:
Ncov =		# entries in covariance parameters table
CovFewer = 	name of covparms table for nested model
NameFewer = label to use for nested model 
CovMore = 	name of covparms table for comparison model
NameMore = 	label to use for comparison model;

%MACRO PseudoR2(NCov, CovFewer, NameFewer, CovMore, NameMore);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&NameFewer."; RUN;
DATA &CovMore.;  LENGTH Name $30.; SET &CovMore.;  Name="&NameMore.";  RUN;
DATA CovCompare; LENGTH Name $30.; SET &CovFewer. &CovMore.; RUN;
DATA CovCompare; SET CovCompare; 
	PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare; 
	IF CovParm IN("UN(2,1)","UN(3,1)","UN(4,1)","UN(3,2)","UN(4,2)","UN(4,3)") 
	THEN DELETE; RUN;
TITLE2 "PsuedoR2 (% Reduction) for &NameFewer. vs. &NameMore.";
PROC PRINT NOOBS DATA=CovCompare; RUN; TITLE2;
%MEND PseudoR2;

* To use FitTest macro:
FitFewer = 	name of infocrit table for nested model
NameFewer = label to use for nested model 
FitMore = 	name of infocrit table for comparison model
NameMore = 	label to use for comparison model;

%MACRO FitTest(FitFewer, NameFewer, FitMore, NameMore);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&NameFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&NameMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
	DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
	DROP AICC HQIC CAIC; RUN;
TITLE2 "Likelihood Ratio Test for &NameFewer. vs. &NameMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE2;
%MEND FitTest;
*******************************************************************************************;

/*** 	Note: These models assume a stacked data structure in which each row
     	provides the response time for a single subject and a single item. ***/

* SAS: Bringing in data from folder to work library;
* Adding another version of predictors to be coded 0/1 for low/high;
DATA Example8a; SET folder.Example8a; 
	IF freq=-.5 THEN freq01=0; IF freq=.5 THEN freq01=1;
	IF size=-.5 THEN size01=0; IF size=.5 THEN size01=1;
run;
TITLE "Crosstabs for item types";
PROC FREQ DATA=Example8a; TABLE freq01*size01; RUN; TITLE;

TITLE "Empty Means Model: No Random Intercepts (E only)";
PROC MIXED DATA=Example8a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
	CLASS Subject Item;
	MODEL rt = / SOLUTION DDFM=Satterthwaite; 
	ODS OUTPUT InfoCrit=FitEonly; * Save fit for comparison;
RUN; TITLE;

TITLE "Random Intercept for Subjects Model";
PROC MIXED DATA=Example8a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
	CLASS Subject Item;
	MODEL rt = / SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT /SUBJECT=Subject TYPE=UN;  * Level 2 variance for subjects;
	ODS OUTPUT InfoCrit=FitRandSub; * Save fit for comparison;
RUN; TITLE;
* Calculate difference in model fit relative to e-only model;
%FitTest(FitFewer=FitEonly, NameFewer=Eonly, FitMore=FitRandSub, NameMore=RandomSubjects);

TITLE "Random Intercepts for Subjects and Items: Crossed Model";
PROC MIXED DATA=Example8a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
	CLASS Subject Item;
	MODEL rt =  / SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT / SUBJECT=Item TYPE=UN; 	* Level 2 variance for items;
	RANDOM INTERCEPT / SUBJECT=Subject TYPE=UN; * Level 2 variance for subjects; 
	ODS OUTPUT InfoCrit=FitRandItem CovParms=CovEmpty; * Save fit, variances to compare;
RUN; TITLE;
* Calculate difference in model fit relative to random subjects model;
%FitTest(FitFewer=FitRandSub, NameFewer=RandomSubjects, FitMore=FitRandItem, NameMore=RandomItems);


TITLE1 "Random Subjects by Random Items Crossed Predictive Model";
TITLE2 "Freq01 and Size01 are not on CLASS statement, so are continuous";
PROC MIXED DATA=Example8a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
	CLASS Subject Item;
	* | operator estimates all possible main effects and interactions up to @ order;
	MODEL rt = freq01|size01@2 / SOLUTION DDFM=Satterthwaite OUTPM=ItemPred;
	RANDOM INTERCEPT / SUBJECT=Item TYPE=UN; 	* Level 2 variance for items;
	RANDOM INTERCEPT / SUBJECT=Subject TYPE=UN; * Level 2 variance for subjects; 
	ODS OUTPUT InfoCrit=FitItem CovParms=CovItem; * Save fit, variances to compare;
* Getting cell means (traditional for Regression);
	ESTIMATE "RT for Low Freq, Small Size"	intercept 1 freq01 0 size01 0 freq01*size01 0;
	ESTIMATE "RT for Low Freq, Large Size"	intercept 1 freq01 0 size01 1 freq01*size01 0;
	ESTIMATE "RT for High Freq, Small Size"	intercept 1 freq01 1 size01 0 freq01*size01 0;
	ESTIMATE "RT for High Freq, Large Size"	intercept 1 freq01 1 size01 1 freq01*size01 1;
* Getting marginal means (traditional for ANOVA);
	ESTIMATE "RT for Low Freq"		intercept 1 freq01  0 size01 .5 freq01*size01  0;
	ESTIMATE "RT for High Freq"		intercept 1 freq01  1 size01 .5 freq01*size01 .5;
	ESTIMATE "RT for Small Size"	intercept 1 freq01 .5 size01  0 freq01*size01  0;
	ESTIMATE "RT for Large Size"	intercept 1 freq01 .5 size01  1 freq01*size01 .5;
	ESTIMATE "Grand Mean for All"	intercept 1	freq01 .5 size01 .5 freq01*size01 .25;
* Getting all possible simple effects (more useful);
	ESTIMATE "Simple Freq Effect for Small Size"	freq01 1 freq01*size01 0;
	ESTIMATE "Simple Freq Effect for Large Size"	freq01 1 freq01*size01 1;
	ESTIMATE "Simple Size Effect for Low Freq"		size01 1 freq01*size01 0;
	ESTIMATE "Simple Size Effect for High Freq"		size01 1 freq01*size01 1;
* Getting all possible marginal effects (traditional for ANOVA, less useful);
	ESTIMATE "Marginal Freq Effect"	freq01 1 freq01*size01 .5;
	ESTIMATE "Marginal Size Effect"	size01 1 freq01*size01 .5;
RUN; TITLE2;
* Get total R2;
PROC CORR DATA=ItemPred; VAR pred rt; RUN;
* Calculate PseudoR2 relative to empty means model;
%PseudoR2(NCov=3, CovFewer=CovEmpty, NameFewer=EmptyMeans, CovMore=CovItem, NameMore=ItemEffects);

TITLE2 "Using CLASS statement to get cell means and comparisons VIA LSMEANS";
PROC MIXED DATA=Example8a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
	* Add freq and size to CLASS statement to use LSMEANS;
	CLASS Subject Item freq01 size01;
	* | operator estimates all possible main effects and interactions up to @ order;
	MODEL rt = freq01|size01@2 / SOLUTION DDFM=Satterthwaite OUTPM=ItemPred;
	RANDOM INTERCEPT / SUBJECT=Item TYPE=UN; 	* Level 2 variance for items;
	RANDOM INTERCEPT / SUBJECT=Subject TYPE=UN; * Level 2 variance for subjects; 
	* Requesting marginal means per condition (what Type 3 tests are for);
	LSMEANS freq01 size01; 
	* Requesting F-tests for simple main effects (more useful than marginal);
	LSMEANS freq01*size01 / SLICE=freq01 SLICE=size01;
RUN; TITLE1; TITLE2;

TITLE1 "Dropping Random Item Intercept";
PROC MIXED DATA=Example8a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
	CLASS Subject Item;
	MODEL rt = freq01|size01@2 / SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT / SUBJECT=Subject TYPE=UN; * Level 2 variance for subjects ONLY; 
	ODS OUTPUT InfoCrit=FitNoRandItem;          * Save fit to compare;
RUN; TITLE1;
* Calculate difference in model fit relative to random subjects and items predictive model;
%FitTest(FitFewer=FitNoRandItem, NameFewer=NoRandomItems, FitMore=FitItem, NameMore=RandomItems);

TITLE1 "Random Slope for Effect of Freq over Subjects";
PROC MIXED DATA=Example8a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
	CLASS Subject Item;
	MODEL rt = freq01|size01@2 / SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT        / SUBJECT=Item TYPE=UN;		* Level 2 variance for items;
	RANDOM INTERCEPT freq01 / SUBJECT=Subject TYPE=UN; 	* Level 2 variances for subjects; 
	ODS OUTPUT InfoCrit=FitRandFreq;         		    * Save fit to compare;
RUN; TITLE1; 
* Calculate difference in model fit relative to random subjects and items predictive model;
%FitTest(FitFewer=FitItem, NameFewer=RandomItems, FitMore=FitRandFreq, NameMore=RandomFreq);

TITLE1 "Random Slope for Effect of Size over Subjects";
PROC MIXED DATA=Example8a COVTEST NOCLPRINT NAMELEN=100 IC METHOD=REML;
	CLASS Subject Item;
	MODEL rt = freq01|size01@2 / SOLUTION DDFM=Satterthwaite;
	RANDOM INTERCEPT        / SUBJECT=Item TYPE=UN;		* Level 2 variance for items;
	RANDOM INTERCEPT size01 / SUBJECT=Subject TYPE=UN; 	* Level 2 variances for subjects; 
	ODS OUTPUT InfoCrit=FitRandSize;         		    * Save fit to compare;
RUN; TITLE1; 
* Calculate difference in model fit relative to random subjects and items predictive model;
%FitTest(FitFewer=FitItem, NameFewer=RandomItems, FitMore=FitRandSize, NameMore=RandomSize);

